use std::sync::Arc;

use crate::entity::la_article;
use crate::entity::la_article_category;
use crate::params::article::{ArticleDetailParams, ArticleListParams};
use crate::response::article::{ArticleDetail, ArticleListItem, SiblingArticle};
use crate::response::common::{BaseResponse, PageResponse};
use crate::state::app::AppState;
use axum::http::StatusCode;
use axum::Json;
use chrono::{TimeZone, Utc};
use sea_orm::{ColumnTrait, EntityTrait, PaginatorTrait, QueryFilter, QueryOrder, QueryTrait};


pub struct ArticleService {
    pub state: Arc<AppState>,
}

impl ArticleService {
    pub fn new(state: &Arc<AppState>) -> Self {
        Self {
            state: Arc::clone(state),
        }
    }

    pub async fn article_list(
        &self,
        params: ArticleListParams,
    ) -> Result<Json<BaseResponse<PageResponse<ArticleListItem>>>, Json<BaseResponse<Vec<u8>>>>
    {
        let page_no = params.page_no.unwrap_or(1);
        let page_size = params.page_size.unwrap_or(20);
        let articles_selection = la_article::Entity::find()
            .filter(la_article::Column::IsDelete.ne(1))
            .filter(la_article::Column::IsShow.ne(0))
            .apply_if(Some(params.cid), |mut query, v| {
                match v {
                    Some(cid) => {
                        query.filter(la_article::Column::Cid.eq(cid))
                    }
                    None => { query }
                }
            });
        let count = match articles_selection.clone().count(&self.state.db_conn).await {
            Ok(total) => total,
            Err(_) => 0,
        };

        let articles_result = articles_selection
            .apply_if(Some(params.sort), |mut query, v| {
                match v {
                    Some(sort_str) => {
                        // 最热
                        if sort_str == "hot" {
                            query
                                .order_by_desc(la_article::Column::Visit)
                                .order_by_desc(la_article::Column::Id)
                        }
                        // 最新
                        else if sort_str == "new" {
                            query.order_by_desc(la_article::Column::Id)
                        } else {
                            query
                                .order_by_desc(la_article::Column::Sort)
                                .order_by_desc(la_article::Column::Id)
                        }
                    }
                    None => query,
                }
            })
            .paginate(&self.state.db_conn, params.page_size.unwrap_or(20))
            .fetch_page(params.page_no.unwrap_or(1) - 1)
            .await;
        match articles_result {
            Ok(articles) => {
                let mut article_list: Vec<ArticleListItem> = vec![];
                for article in articles {
                    let dt = Utc.timestamp_opt(article.create_time as i64, 0);
                    let article_item = ArticleListItem {
                        id: article.id,
                        title: article.title,
                        image: Some(article.image),
                        intro: Some(article.intro),
                        visit: article.visit,
                        collect: false,
                        create_time: dt.unwrap().format("%Y-%m-%d %H:%M:%S").to_string(),
                    };
                    article_list.push(article_item);
                }
                let page_data = PageResponse {
                    count,
                    page_no,
                    page_size,
                    lists: article_list,
                };
                let response = BaseResponse {
                    code: StatusCode::OK.as_u16(),
                    message: "成功".to_string(),
                    data: page_data,
                };
                Ok(Json(response))
            }
            Err(err) => {
                let err_resp = BaseResponse {
                    code: StatusCode::INTERNAL_SERVER_ERROR.as_u16(),
                    message: format!("{}", err),
                    data: vec![],
                };
                Err(Json(err_resp))
            }
        }
    }

    pub async fn article_detail(
        &self,
        params: ArticleDetailParams,
    ) -> Result<Json<BaseResponse<ArticleDetail>>, Json<BaseResponse<Vec<u8>>>> {
        let id: u32 = match params.id.parse() {
            Ok(id) => id,
            Err(_) => {
                let err_resp = BaseResponse {
                    code: StatusCode::INTERNAL_SERVER_ERROR.as_u16(),
                    message: "请输入正确的文章 id".to_string(),
                    data: vec![],
                };
                return Err(Json(err_resp));
            }
        };

        match la_article::Entity::find_by_id(id).find_also_related(la_article_category::Entity)
            .filter(la_article::Column::IsDelete.eq(0))
            .filter(la_article::Column::IsShow.eq(1))
            .one(&self.state.db_conn)
            .await
        {
            Ok(Some((article, category))) => {
                // 上一篇文章
                let prev_result = la_article::Entity::find()
                    .filter(la_article::Column::IsDelete.eq(0))
                    .filter(la_article::Column::IsShow.eq(1))
                    .filter(la_article::Column::Id.lt(article.id))
                    .order_by_desc(la_article::Column::Sort)
                    .order_by_desc(la_article::Column::Id)
                    .one(&self.state.db_conn)
                    .await;
                let prev = match prev_result {
                    Ok(Some(article)) => Some(SiblingArticle {
                        id: article.id,
                        title: article.title,
                    }),
                    _ => None
                };
                let next_result = la_article::Entity::find()
                    .filter(la_article::Column::IsDelete.eq(0))
                    .filter(la_article::Column::IsShow.eq(1))
                    .filter(la_article::Column::Id.gt(article.id))
                    .order_by_asc(la_article::Column::Sort)
                    .order_by_asc(la_article::Column::Id)
                    .one(&self.state.db_conn)
                    .await;
                let next = match next_result {
                    Ok(Some(article)) => Some(SiblingArticle {
                        id: article.id,
                        title: article.title,
                    }),
                    _ => None
                };
                let article_detail = ArticleDetail {
                    id: article.id,
                    cid: Option::from(article.cid),
                    category: match category {
                        Some(category) => Some(category.name),
                        None => None
                    },
                    title: article.title,
                    intro: Option::from(article.intro),
                    summary: article.summary,
                    image: Option::from(article.image),
                    content: article.content,
                    author: article.author,
                    visit: article.visit,
                    sort: article.sort,
                    is_collect: 0,
                    create_time: "".to_string(),
                    update_time: None,
                    prev,
                    next,
                };
                let resp = BaseResponse {
                    code: StatusCode::OK.as_u16(),
                    message: "成功".to_string(),
                    data: article_detail,
                };
                Ok(Json(resp))
            }
            Ok(None) => {
                let err_resp = BaseResponse {
                    code: StatusCode::MULTIPLE_CHOICES.as_u16(),
                    message: "文章数据不存在!".to_string(),
                    data: vec![],
                };
                Err(Json(err_resp))
            }
            Err(err) => {
                let err_resp = BaseResponse {
                    code: StatusCode::INTERNAL_SERVER_ERROR.as_u16(),
                    message: format!("{}", err),
                    data: vec![],
                };
                Err(Json(err_resp))
            }
        }
    }
}
